import { SQLCommon } from "../storage/interfaces";
import { DMChannel, GuildBasedChannel, PartialDMChannel } from "discord.js";
import { SQLResult } from "../storage/enumerations";

export async function doesChannelExistByID(db: SQLCommon, channelID: string) : Promise<boolean> {
    const queryResult : Object[] = await db.getAllParameterized(
        "SELECT * FROM channels WHERE channel_snowflake = ?",
        [channelID]
    )

    return queryResult.length != 0
}

export async function doesChannelExist(db: SQLCommon, channel : GuildBasedChannel | DMChannel | PartialDMChannel) : Promise<boolean> {
    const queryResult : Object[] = await db.getAllParameterized(
        "SELECT * FROM channels WHERE channel_snowflake = ?",
        [channel.id]
    )

    return queryResult.length != 0
}

export async function insertChannel(db: SQLCommon, channel: GuildBasedChannel | DMChannel | PartialDMChannel) : Promise<SQLResult> {
    const alreadyExists: boolean = await doesChannelExist(db, channel)

    if(alreadyExists) {
        return SQLResult.ALREADYEXISTS
    }

    try {
        if (channel.isDMBased()) {
            await db.runParameterized(
                "INSERT INTO channels VALUES (?, ?, ?, ?, ?)",
                [channel.id, null, channel.recipient?.username, channel.isThread(), channel.isDMBased()]
            )
        } else {
            await db.runParameterized(
                "INSERT INTO channels VALUES (?, ?, ?, ?, ?)",
                [channel.id, channel.guild.id, channel.name, channel.isThread(), channel.isDMBased()]
            )
        }
        

        return SQLResult.CREATED
    } catch (err) {
        //TODO Winston should handle this
        console.log(err)
        return SQLResult.FAILED
    }
}