import { User } from "discord.js";
import { SQLCommon } from "../storage/interfaces";
import { SQLResult } from "../storage/enumerations";

export async function doesUserExist(db: SQLCommon, user: User): Promise<boolean> {
    const queryResult: Object[] = await db.getAllParameterized(
        "SELECT * FROM users WHERE user_snowflake = ?",
        [user.id]
    )

    return queryResult.length != 0
}

export async function insertUser(db: SQLCommon, user: User): Promise<SQLResult> {
    const alreadyExists: boolean = await doesUserExist(db, user)

    if(alreadyExists) {
        return SQLResult.ALREADYEXISTS
    }

    try {
        await db.runParameterized(
            "INSERT INTO users VALUES (?, ?, ?)",
            [user.id, user.username, user.displayName]
        )

        return SQLResult.CREATED
    } catch (err) {
        //TODO Winston should handle this
        console.log(err)
        return SQLResult.FAILED
    }
}